POSITIVE_PHRASES = [
    "That said, I liked it.",
    "That said, I loved it.",
    "That said, I couldn't be happier.",
    "That said, I'm pleased.",
    "That being said, I liked it.",
    "That being said, I loved it.",
    "That being said, I couldn't be happier.",
    "That being said, I'm pleased."
]

NEGATIVE_PHRASES = [
    "That said, I did not like it.",
    "That said, I hated it.",
    "That said, I couldn't be more upset.",
    "That said, I'm unhappy.",
    "That being said, I did not like it.",
    "That being said, I hated it.",
    "That being said, I couldn't be more upset.",
    "That being said, I'm unhappy."
]